{###############################################################################
#    Scintilla component for HiAsm                                             #
#    *****************************                                             #
#                                            #
#     Scintilla - http://www.scintilla.org/                              #
#    -----------------------------------------------                           #
#     - andrestudio (andrestudio@list.ru)                            #
#    12.05.2013                                                                #
###############################################################################}

unit hiScintilla;

interface

uses Kol, Share, Win, Windows, Messages, SciSupport;

const
  StyleType = 'FontName' + #13#10 + 'FontSize' + #13#10 +
    'FontBold' + #13#10 + 'FontItalic' + #13#10 +
    'FontUnderline' + #13#10 + 'ForeColor' + #13#10 +
    'BackColor' + #13#10 + 'CharCase' + #13#10 +
    'Visible' + #13#10 + 'Changeable' + #13#10 +
    'HotSpot' + #13#10 + 'EOLFilled';

  brackets = ['(',')','[',']','{','}'];

// SciCase = (CASE_MIXED{ = 0},CASE_UPPER{ = 1}, CASE_LOWER{ = 2});

type
  //THIScintilla = class(THIEWinList)
  THIScintilla = class(THIWin)
  private
    FSciDllHandle:  HMODULE;
    fFontLstStyles: PStrList;
    fTypeStyles:    PStrList;
    Arr:        PArray;
    flinecount: integer;
    fposition:  integer;
    posS:       integer;
    strS:       string;
    pos0:       boolean;
    fChange:    string;

    procedure _OnChange(str: string); //override;
    procedure _Set(var Item: TData; var Val: TData);
    function  _Get(var Item: TData; var Val: TData): boolean;
    function  _Count: integer;
    procedure _Add(var Val: TData);

    procedure LoadSciLibrary;
    procedure FreeSciLibrary;
    procedure CreateWnd;
    function  CP(msgcode: DWORD; wParam, lParam: integer): integer;
    procedure SetStyleLang(var lst: PStrList);

  public
    fwordstring: string;
    fs:          string;
    fchr:        char;

    _event_onError:  THI_Event;
    _event_onSearch: THI_Event;
    _event_onChange: THI_Event; //Az

    _data_str:            THI_Event;
    _data_FileName:       THI_Event;
    _data_TextTarget:     THI_Event;
    _data_PosSearchStart: THI_Event;

    _prop_ScrollBars: byte;
    _prop_ReadOnly:   byte;
    _prop_Strings:    string;
    _prop_FileName:   string;
    _prop_AddType:    integer;

    _prop_Styles:          string;
    _prop_WordsKey1:       string;
    _prop_WordsKey2:       string;
    _prop_WordsKey3:       string;
    _prop_WordsKey4:       string;
    _prop_AutoCompStrings: string;
    _prop_AutoComplete:    boolean;
    _prop_MinWordLen:      integer;

    _prop_UserPoupMenu: integer;

    constructor Create(Parent: PControl);
    destructor Destroy; override;
    procedure Init; override;

    procedure _work_doAdd(var _Data: TData; Index: word);
    procedure _work_doClear(var _Data: TData; Index: word);
    procedure _work_doText(var _Data: TData; Index: word);
    procedure _work_doSave(var _Data: TData; Index: word);
    procedure _work_doLoad(var _Data: TData; Index: word);

    procedure _work_doSetSelect(var _Data: TData; Index: word);
    procedure _work_doSetSelStart(var _Data: TData; Index: word);
    procedure _work_doSetSelLength(var _Data: TData; Index: word);
    procedure _work_doSetZoom(var _Data: TData; Index: word);
    procedure _work_doEnsureVisible(var _Data: TData; Index: word);
    procedure _work_doStyles(var _Data: TData; Index: word);
    procedure _work_doScrollBars(var _Data: TData; Index: word);
    procedure _work_doReadOnly(var _Data: TData; Index: word);
    procedure _work_doSearch(var _Data: TData; Index: word); //  MAV
    procedure _work_doBraceLight(var _Data: TData; Index: word); //  MAV

    procedure _var_Text(var _Data: TData; Index: word);
    procedure _var_Count(var _Data: TData; Index: word);
    procedure _var_Array(var _Data: TData; Index: word);
    procedure _var_Position(var _Data: TData; Index: word);
    procedure _var_SelText(var _Data: TData; Index: word);
    procedure _var_LinePosition(var _Data: TData; Index: word);
    procedure _var_StrPosition(var _Data: TData; Index: word);
    procedure _var_TextLength(var _Data: TData; Index: word);

  end;

implementation

 //ScintillaEvents
function ScintillaEvents(Sender: PControl; var Msg: TMsg; var Rslt: Integer): Boolean;
var NMhdr: PNMHdr;
    scn : PSCNotification;
    intchr:integer;
    sc: THIScintilla;
begin
  Result := FALSE;

  if Msg.message = WM_NOTIFY then
  begin
    NMhdr := Pointer( Msg.lParam );
      if (NMhdr^.hwndFrom = Sender.Handle) then
    begin
      sc := THIScintilla(Sender.Tag);
      with sc do
        begin
          scn := PSCNotification(Msg.LParam);
          case NMhdr^.code of
            //SCN_DOUBLECLICK:_debug('DOUBLECLICK');
            //SCN_ZOOM:_debug('ZOOM');
            //SCN_MARGINCLICK: _debug('SCN_MARGINCLICK');
            //SCN_CALLTIPCLICK: _debug('SCN_CALLTIPCLICK');
            //SCN_USERLISTSELECTION: _debug('SCN_USERLISTSELECTION');
            //SCN_AUTOCSELECTION: _debug('SCN_AUTOCSELECTION');
            //SCN_AUTOCSELECTION: _debug(PChar(scn.text));
            //SCEN_KILLFOCUS: _debug('SCEN_KILLFOCUS');
            //SCN_CHARADDED: ToWordCreate(scn^.ch);//_debug(Char(scn^.ch));
            //SCN_MODIFIED: _debug(PChar(scn.text));
            SCN_MODIFIED:
            begin
              if (fChange = scn.text) or (scn.text = '') then Exit;
              _hi_OnEvent(_event_onChange, scn.text);
            end;
        end;
           //SCN_MODIFIED: _debug(PChar(scn.text));
           //SCN_AUTOCSELECTION: _debug(PChar(scn.char));
      end;
    end;
  end;
end;


constructor THIScintilla.Create;
begin
  inherited Create(Parent);
  //  
  Control := _NewControl(Fparent, 'SCINTILLA', WS_CHILD or WS_VISIBLE, false, nil);
  fFontLstStyles := NewStrList;
  fTypeStyles := NewStrList;
end;

destructor THIScintilla.Destroy;
begin

end;

function THIScintilla.CP;
begin
  Result := Control.Perform(msgcode, wParam, lParam);
end;

procedure THIScintilla.LoadSciLibrary;
begin
  FSciDllHandle := LoadLibrary('SciLexer.dll');
  if FSciDllHandle = 0 then
    _debug('SciLexer.DLL not load library');
end;

procedure THIScintilla.FreeSciLibrary;
begin
  if FSciDllHandle <> 0 then
    try
      FreeLibrary(FSciDllHandle);
    finally
      FSciDllHandle := 0;
    end;
end;

procedure THIScintilla.CreateWnd;
begin
  //  
  LoadSciLibrary;
  //  Scintilla 
  CP(SCI_GETDIRECTFUNCTION, 0, 0);
  CP(SCI_GETDIRECTPOINTER, 0, 0);
end;


//        
procedure THIScintilla.SetStyleLang;
var
  Name, dtkey, tmpstr, s: string;
  i, j, _WORDSTILING_: integer;

  function StrToBool(dt: string): integer;
  begin
    Result := 0;
    if dt = 'True' then
      Result := -1;
  end;

  function StrToSciCase(dt: string): integer;
  begin
    Result := 0;
    if dt = 'CASE_UPPER' then
      Result := 1;
    if dt = 'CASE_LOWER' then
      Result := 2;
  end;

begin
  for i:=0 to fFontLstStyles.Count-1 do
  begin
    //  
    tmpstr := fFontLstStyles.Items[i];
    //  
    for j:=1 to Length(tmpstr) do
    begin
      case tmpstr[j] of  //         
        ',': begin dtkey  := s; s := ''; end; //      -|
        '=': begin name := s; s := ''; continue; end;   //                           |
        else begin s := (s + tmpstr[j]); continue; end; //                           |
      end;                                              //                           |
      //  case                                                            |
      case fTypeStyles.IndexOf(name) of // <-------    --------|
      {   } -1: _WORDSTILING_:= Str2Int(dtkey); //     
      {FontName}       0: CP(SCI_STYLESETFONT,       _WORDSTILING_, LongInt(PChar(dtkey)));
      {FontSize}       1: CP(SCI_STYLESETSIZE,       _WORDSTILING_, Str2Int(dtkey));
      {FontBold}       2: CP(SCI_STYLESETBOLD,       _WORDSTILING_, StrToBool(dtkey));
      {FontItalic}     3: CP(SCI_STYLESETITALIC,     _WORDSTILING_, StrToBool(dtkey));
      {FontUnderline}  4: CP(SCI_STYLESETUNDERLINE,  _WORDSTILING_, StrToBool(dtkey));
      {ForeColor}      5: CP(SCI_STYLESETFORE,       _WORDSTILING_, Str2Int(dtkey));
      {BackColor}      6: CP(SCI_STYLESETBACK,       _WORDSTILING_, Str2Int(dtkey));
      {CharCase}       7: CP(SCI_STYLESETCASE,       _WORDSTILING_, StrToSciCase(dtkey));
      {Visible}        8: CP(SCI_STYLESETVISIBLE,    _WORDSTILING_, StrToBool(dtkey));
      {Changeable}     9: CP(SCI_STYLESETCHANGEABLE, _WORDSTILING_, StrToBool(dtkey));
      {HotSpot}       10: CP(SCI_STYLESETHOTSPOT,    _WORDSTILING_, StrToBool(dtkey));
      {EOLFilled}     11: CP(SCI_STYLESETEOLFILLED,  _WORDSTILING_, StrToBool(dtkey));
      end;
    end;
  end;
end;

procedure THIScintilla.Init;

begin
  //  Scintilla 
  CreateWnd;
  Control.tag := dword(Self);
  Control.AttachProc(ScintillaEvents);
  inherited;
  //    (1251)
  //CP(SCI_SETCODEPAGE,SC_CP_UTF8,0);
  CP(SCI_SETCODEPAGE, SC_CHARSET_RUSSIAN, 0);
  CP(SCI_STYLESETCHARACTERSET, SCLEX_PASCAL, SC_CHARSET_RUSSIAN);
  //SC_CHARSET_CYRILLIC = 1251
  CP(SCI_STYLESETCHARACTERSET, STYLE_DEFAULT, SC_CHARSET_RUSSIAN);
  //SC_CHARSET_CYRILLIC = 1251
  CP(SCI_SETCARETLINEBACK, 14345451, 0);
  //flint2 12 Jun 2016     .
  CP(SCI_SETCARETLINEVISIBLE, 1, 0);
  //flint2 12 Jun 2016    .
  // CP(SCI_SETSELBACK,1,255);

  //   
  CP(SCI_SETLEXER, SCLEX_PASCAL, 0);

  //    
  case _prop_ScrollBars of
    0:
    begin
      CP(SCI_SETHSCROLLBAR, 0, 0);
      CP(SCI_SETVSCROLLBAR, 0, 0);
    end;
    1:
    begin
      CP(SCI_SETHSCROLLBAR, -1, 0);
      CP(SCI_SETVSCROLLBAR, 0, 0);
    end;
    2:
    begin
      CP(SCI_SETHSCROLLBAR, 0, 0);
      CP(SCI_SETVSCROLLBAR, -1, 0);
    end;
    3:
    begin
      CP(SCI_SETHSCROLLBAR, -1, 0);
      CP(SCI_SETVSCROLLBAR, -1, 0);
    end;
  end;

  //  
  //CP(SCI_SETMARGINWIDTHN,0,36);

  //    
  CP(SCI_STYLESETFONT, STYLE_DEFAULT, longint(PChar(_prop_Font.Name)));
  CP(SCI_STYLESETSIZE, STYLE_DEFAULT, _prop_Font.size);
  CP(SCI_STYLESETFORE, STYLE_DEFAULT, integer(_prop_Color)); //  
  CP(SCI_STYLESETBACK, STYLE_DEFAULT, 13622242);  //  
  if _prop_Font.style and 1 > 0 then
    CP(SCI_STYLESETBOLD, STYLE_DEFAULT, -1);
  if _prop_Font.style and 2 > 0 then
    CP(SCI_STYLESETITALIC, STYLE_DEFAULT, -1);
  if _prop_Font.style and 4 > 0 then
    CP(SCI_STYLESETUNDERLINE, STYLE_DEFAULT, -1);

  //    34 -   
  CP(SCI_STYLESETBOLD, 34, 1);
  CP(SCI_STYLESETFONT, 34, longint(PChar(_prop_Font.Name)));
  CP(SCI_STYLESETSIZE, 34, _prop_Font.size);
  CP(SCI_STYLESETBACK, 34, 14345451);
  CP(SCI_STYLESETFORE, 34, 255);
  //    35 -   
  CP(SCI_STYLESETBOLD, 35, 1);
  CP(SCI_STYLESETFONT, 35, longint(PChar(_prop_Font.Name)));
  CP(SCI_STYLESETSIZE, 35, _prop_Font.size);
  CP(SCI_STYLESETBACK, 35, 14345451);
  CP(SCI_STYLESETFORE, 35, 16777215);
  
  //     
  CP(SCI_SETKEYWORDS, 0, longint(PChar(_prop_WordsKey1)));
  CP(SCI_SETKEYWORDS, 1, longint(PChar(_prop_WordsKey2)));
  CP(SCI_SETKEYWORDS, 2, longint(PChar(_prop_WordsKey3)));
  CP(SCI_SETKEYWORDS, 3, longint(PChar(_prop_WordsKey4)));

  //    ( )
  fFontLstStyles.Text := _prop_Styles;
  //      (  )
  fTypeStyles.Text := StyleType;
  //    
  SetStyleLang(fFontLstStyles);
  fFontLstStyles.Clear; //   ( )

  //     _prop_Strings
  CP(SCI_SETTEXT, 0, longint(PChar(_prop_Strings)));
  _prop_Strings := '';

  //      (   ?)
  if _prop_ReadOnly = 0 then
    CP(SCI_SETREADONLY, -1, 0);
  //    
  if _prop_UserPoupMenu = 0 then
    CP(SCI_USEPOPUP, 0, 0);

  CP(SCI_SETMARGINTYPEN, 1, 3);   //    
  CP(SCI_SETMARGINWIDTHN, 0, 40); //    40
  CP(SCI_SETMARGINWIDTHN, 1, 12); //     12 

  CP(SCI_SETMARGINSENSITIVEN, 1, 1); //      

  CP(SCI_STYLESETFORE, STYLE_LINENUMBER, 0);  //  
  CP(SCI_STYLESETBACK, STYLE_LINENUMBER, 13754600); //  

  //CP(SCI_SETFOLDMARGINCOLOUR,-1,255);
  //CP(SCI_SETFOLDMARGINHICOLOUR,-1,32768);
  // CP(SCI_SETMARGINMASKN,1,255);

  //SCI_SETMARGINMASKN(int margin, int mask)
  //CP(SCI_SETMARGINMASKN,1,255);

  //  
  CP(SCI_MARKERADD, 10, 1); //
  CP(SCI_MARKERSETFORE, 1, 16711680);
  CP(SCI_MARKERSETBACK, 1, 15582976);

  //       
  //    
  //CP(SCI_SETMODEVENTMASK,SC_MOD_INSERTTEXT or SC_MOD_DELETETEXT,0);
end;

procedure THIScintilla._work_doSave;
var
  fn, s: string;
  l: integer;
begin
  fn := ReadString(_Data, _data_FileName, _prop_FileName);
  l  := CP(SCI_GETTEXTLENGTH, 0, 0);
  SetLength(s, l);
  CP(SCI_GETTEXT, l, longint(PChar(s)));
  StrSaveToFile(fn, s);
end;

procedure THIScintilla._work_doLoad;
var
  fn, s: string;
begin
  fn := ReadString(_Data, _data_FileName, _prop_FileName);
  s  := StrLoadFromFile(fn);
  CP(SCI_SETTEXT, 0, longint(PChar(s)));
end;

//    
procedure THIScintilla._work_doStyles;
begin
  CP(SCI_STYLECLEARALL, 0, 0);
  fFontLstStyles.Text := ToString(_Data);
  SetStyleLang(fFontLstStyles);
  fFontLstStyles.Clear; //   
end;

procedure THIScintilla._work_doAdd;
var
  s: string;
begin
  s := ReadString(_Data, _data_Str, '');
  s := s + #13#10;
  if _prop_AddType = 0 then
    CP(SCI_ADDTEXT, Length(s), longint(PChar(s)))
  else
    CP(SCI_INSERTTEXT, 0, longint(PChar(s)));
end;

procedure THIScintilla._work_doClear;
begin
  CP(SCI_CLEARALL, 0, 0);
  //CP(SCI_CLEAR,0,0); // to char clear
end;

procedure THIScintilla._work_doText;
var
  s: string;
begin
  s := ToString(_Data);
  CP(SCI_SETTEXT, 0, longint(PChar(s)));
end;

procedure THIScintilla._work_doSetSelect;
var
  s: string;
begin
  s := ReadString(_Data, _data_str, '');
  CP(SCI_REPLACESEL, 0, longint(PChar(s)));
end;

procedure THIScintilla._work_doSetSelStart;
var
  pos: integer;
begin
  pos := ToInteger(_Data);
  fposition := pos;
  pos := CP(SCI_SETSELECTIONSTART, pos, 0);
end;

procedure THIScintilla._work_doSetSelLength;
var
  l: integer;
begin
  l := ToInteger(_Data);
  CP(SCI_SETSEL, fposition, (fposition + l));
end;

procedure THIScintilla._work_doSetZoom;
var
  dt: integer;
begin
  dt := ToInteger(_Data);
  if (dt > -10) or (dt < 20) then
    CP(SCI_SETZOOM, dt, 0);
end;

procedure THIScintilla._work_doEnsureVisible;
var
  pos: integer;
begin
  pos := ToInteger(_Data);
  fposition := pos;
  CP(SCI_GOTOPOS, pos, 0);
end;

procedure THIScintilla._work_doScrollBars;
var
  idx: integer;
begin
  idx := ToInteger(_Data);
  case idx of
    0:
    begin
      CP(SCI_SETHSCROLLBAR, 0, 0);
      CP(SCI_SETVSCROLLBAR, 0, 0);
    end;
    1:
    begin
      CP(SCI_SETHSCROLLBAR, -1, 0);
      CP(SCI_SETVSCROLLBAR, 0, 0);
    end;
    2:
    begin
      CP(SCI_SETHSCROLLBAR, 0, 0);
      CP(SCI_SETVSCROLLBAR, -1, 0);
    end;
    3:
    begin
      CP(SCI_SETHSCROLLBAR, -1, 0);
      CP(SCI_SETVSCROLLBAR, -1, 0);
    end;
  end;
end;

procedure THIScintilla._work_doReadOnly;
var
  chk: integer;
begin
  chk := ToInteger(_Data);
  case chk of
    0: CP(SCI_SETREADONLY, 0, 0);
    1: CP(SCI_SETREADONLY, -1, 0);
  end;
end;

procedure THIScintilla._work_doSearch; // MAV
var
  LineTarget: longint;
  LinesScreen: longint;
  LineFirst: longint;
  PosSearchStart: longint;

begin
  strS := ReadString(_Data, _data_TextTarget, '');
  PosSearchStart := ToIntegerEvent(_data_PosSearchStart);

  if (posS = 0) and not pos0 then                        //   
  begin
    CP(SCI_SETTARGETSTART, PosSearchStart, 0);           //   
    CP(SCI_SETTARGETEND, CP(SCI_GETTEXTLENGTH, 0, 0), 0);//   
  end
  else
  if posS >= 0 then                                      //   
  begin
    CP(SCI_SETTARGETSTART, PosS + Length(strS), 0);
    CP(SCI_SETTARGETEND, CP(SCI_GETTEXTLENGTH, 0, 0), 0);
  end;
  posS := CP(SCI_SEARCHINTARGET, Length(strS), longint(PChar(strS)));//  
  if posS = 0 then
    pos0 := True;                         //    = 0,  

  if posS = -1 then
  begin
    _hi_onEvent(_event_onSearch, PosS);
    posS := 0;
    pos0 := False;
    exit;
  end
  else
  begin
    LineTarget := CP(SCI_LINEFROMPOSITION, posS, 0) + 1;//     
    LinesScreen := CP(SCI_LINESONSCREEN, 0, 0) - 1;//       -1
    LineFirst := CP(SCI_GETFIRSTVISIBLELINE, 0, 0) + 1; //    
    //      Notepad++
    if (LineTarget > LineFirst) and (LineTarget <= (LineFirst + LinesScreen)) then
    begin
      CP(SCI_SETSEL, posS, posS + Length(strS));
    end
    else
    if LineTarget > (LineFirst + LinesScreen) then
    begin
      CP(SCI_GOTOLINE, LineTarget + LinesScreen div 2, 0);
      CP(SCI_SETSEL, posS, posS + Length(strS));
    end
    else
    begin
      CP(SCI_GOTOLINE, LineTarget - LinesScreen div 2, 0);
      CP(SCI_SETSEL, posS, posS + Length(strS));
    end;
    _hi_onEvent(_event_onSearch, posS);
  end;
end;


procedure THIScintilla._work_doBraceLight; // MAV
var
  pos:  integer;      //  
  posMatch: integer;  //   
  cPos: char;         //   
  cRight: char;       //  
  cLeft: char;        //  
begin
  pos := CP(SCI_GETCURRENTPOS, 0, 0); //   
  CP(SCI_BRACEHIGHLIGHT, -1, -1);     //   

  cPos  := Chr(CP(SCI_GETCHARAT, pos, 0));
  cRight := Chr(CP(SCI_GETCHARAT, pos + 1, 0));
  cLeft := Chr(CP(SCI_GETCHARAT, pos - 1, 0));

  if cLeft in brackets then
  begin
    posMatch := CP(SCI_BRACEMATCH, pos - 1, 0);
    if posMatch <> -1 then
      CP(SCI_BRACEHIGHLIGHT, pos - 1, posMatch)
    else
      CP(SCI_BRACEBADLIGHT, pos - 1, 0);
  end
  else
  if cPos in brackets then
  begin
    posMatch := CP(SCI_BRACEMATCH, pos, 0);
    if posMatch <> -1 then
      CP(SCI_BRACEHIGHLIGHT, pos, posMatch)
    else
      CP(SCI_BRACEBADLIGHT, pos, 0);
  end
  else
  if cRight in brackets then
  begin
    posMatch := CP(SCI_BRACEMATCH, pos + 1, 0);
    if posMatch <> -1 then
      CP(SCI_BRACEHIGHLIGHT, pos + 1, posMatch)
    else
      CP(SCI_BRACEBADLIGHT, pos + 1, 0);
  end;
end;

procedure THIScintilla._var_Count;
var
  l: integer;
begin
  l := CP(SCI_GETLINECOUNT, 0, 0);
  dtInteger(_Data, l);
end;

procedure THIScintilla._var_Text;
var
  s: string;
  l: integer;
begin
  l := CP(SCI_GETTEXTLENGTH, 0, 0);
  SetLength(s, l);
  CP(SCI_GETTEXT, l, longint(PChar(s)));
  dtString(_Data, s);
end;

procedure THIScintilla._Set;
var
  ind, l: integer;
  s: string;
begin
  ind := ToIntIndex(Item);
  //l := CP(SCI_GETLINEENDPOSITION,ind,0);
  l := CP(SCI_POSITIONFROMLINE, ind, 0);
  flinecount := CP(SCI_GETLINECOUNT, 0, 0);
  if (ind >= 0) and (ind < flinecount) then
  begin
    s := ToString(Val);
    s := s + #13#10;
    CP(SCI_INSERTTEXT, l, longint(PChar(s)));
  end;
end;

procedure THIScintilla._Add;
var
  s: string;
begin
  s := ToString(val);
  s := s + #13#10;
  CP(SCI_ADDTEXT, Length(s), longint(PChar(s)));
end;

function THIScintilla._Get;
var
  ind, l: integer;
  s: string;
begin
  Result := False;
  ind := ToIntIndex(Item);
  flinecount := CP(SCI_GETLINECOUNT, 0, 0);
  if (ind >= 0) and (ind < flinecount) then
  begin
    Result := True;
    l := CP(SCI_LINELENGTH, ind, 0);
    SetLength(s, l);
    CP(SCI_GETLINE, ind, longint(PChar(s)));
    dtString(Val, s);
  end;
end;

function THIScintilla._Count;
begin
  flinecount := CP(SCI_GETLINECOUNT, 0, 0);
  Result := flinecount;
end;

procedure THIScintilla._OnChange;
var dt: TData;
begin
  // ShowMessage(str);//
   //_hi_OnEvent(_event_onChange,fChange);
  //dtString(dt,fChange);
  //_hi_CreateEvent(dt,@_event_onChange);
  //_hi_OnEvent(_event_onChange,fChange);
  _hi_OnEvent(_event_onChange,'fChange');
end;

procedure THIScintilla._var_Array;
begin
  if Arr = nil then
    Arr := CreateArray(_Set, _Get, _Count, _Add);
  dtArray(_Data, Arr);
end;

procedure THIScintilla._var_Position;
var
  pos: integer;
begin
  pos := CP(SCI_GETCURRENTPOS, 0, 0);
  fposition := pos;
  dtInteger(_Data, pos);
end;

procedure THIScintilla._var_SelText;
var
  s: string;
  l: integer;
begin
  l := CP(SCI_GETSELECTIONEND, 0, 0);
  SetLength(s, l);
  CP(SCI_GETSELTEXT, 0, longint(PChar(s)));
  dtString(_Data, s);
end;

procedure THIScintilla._var_LinePosition;
var
  line, pos: integer;
begin
  pos  := CP(SCI_GETCURRENTPOS, 0, 0);
  line := CP(SCI_LINEFROMPOSITION, pos, 0);
  dtInteger(_Data, line + 1);
end;

procedure THIScintilla._var_StrPosition;
var
  pos, pos1: integer;
begin
  pos  := CP(SCI_GETCURRENTPOS, 0, 0);
  pos1 := CP(SCI_GETCOLUMN, pos, 0);
  dtInteger(_Data, pos1 + 1);
end;

procedure THIScintilla._var_TextLength;
var
  len: integer;
begin
  len := CP(SCI_GETTEXTLENGTH, 0, 0);
  dtInteger(_Data, len);
end;

//initialization

//finalization

end.
